<?php

namespace NoccyLabs\JsonEdit\Tree;

class Tree
{

    public ?Node $root = null;

    public function load(mixed $document): self
    {
        $this->root = $this->parseNode($document);
        return $this;
    }

    public function save(): mixed
    {
        if (!$this->root) 
            return null;
        
        return json_decode(
            json_encode(
                $this->root
            )
        );
    }

    private function parseNode(mixed $node): Node
    {
        if (is_array($node) && array_is_list($node)) {
            return new ArrayNode(
                array_map($this->parseNode(...), $node)
            );
        } elseif (is_object($node) || is_array($node)) {
            return new ObjectNode(
                array_combine(
                    array_keys((array)$node),
                    array_map($this->parseNode(...), (array)$node)
                )
            );
        } else {
            return new ValueNode($node);
        }
    }

    public function search(string $search): array
    {
        return $this->searchNode(null, $this->root, $search);
    }

    /**
     * 
     * 
     * @return array<Node>
     */
    private function searchNode(?string $key, Node $node, string $search): array
    {
        $results = [];
        
        if ($node instanceof ArrayNode) {
            if ($key && str_contains($key, $search)) {
                $results[] = $node;
            }
            foreach ($node->items as $index=>$item) {
                $results = [ ...$results, ...$this->searchNode($index, $item, $search) ];
            }            
        } elseif ($node instanceof ObjectNode) {
            if ($key && str_contains($key, $search)) {
                $results[] = $node;
            }
            foreach ($node->properties as $key=>$item) {
                $results = [ ...$results, ...$this->searchNode($key, $item, $search) ];
            }            
        } elseif ($node instanceof ValueNode) {
            if (str_contains($node->value, $search) || ($key && str_contains($key, $search))) {
                $results[] = $node;
            }
        }

        return $results;
    }

}

